from scipy.spatial import distance as dist
from collections import OrderedDict
import numpy as np
import cv2


class ColorLabler:
    def __init__(self):
        # initialize the colors dictionary, containing the color
		# name as the key and the RGB tuple as the value
#        colors = ({
#            "Blue": (33.5, 35.5, 150),
#            "Red": (153, 59.5, 27),
#            "Yellow": (206, 217.5, 50),
#            "Cyan": (43, 255, 255),
#            "Green": (61, 165, 85)})
	colors = ({
            "Blue": (28, 70, 152),
            "Red": (126, 34, 47),
            "Yellow": (86, 83, 42),
            "Cyan": (109, 131, 180),
            "Green": (2, 77, 54)})

		# allocate memory for the L*a*b* image, then initialize
		# the color names list
        self.lab = np.zeros((len(colors), 1, 3), dtype = "uint8")
        self.colorNames = []

		# loop over the colors dictionary
        for (i, (name,rgb)) in enumerate(colors.items()):
		# update the L*a*b* array and the color names list
            self.lab[i] = rgb
            self.colorNames.append(name)
		# convert the L*a*b* array from the RGB color space
		# to L*a*b*
        self.lab = cv2.cvtColor(self.lab, cv2.COLOR_RGB2LAB)

    def label(self, image, c):
		# construct a mask for the contour, then compute the
		# average L*a*b* value for the masked region
        mask = np.zeros(image.shape[:2], dtype = "uint8")
        cv2.drawContours(mask, [c], -1, 255, -1)
	#cv2.imwrite("mask1.png",mask)
        mask = cv2.erode(mask, None, iterations = 2)
	#cv2.imwrite("mask2.png", mask)
        mean = cv2.mean(image, mask = mask)[:3]

		# initialize the minimum distance found thus far
        minDist = (np.inf, None)

		# loop over the known L*a*b* color values
        for (i, row) in enumerate(self.lab):
			# compute the distance between the current L*a*b*
			# color value and the mean of the image
            d = dist.euclidean(row[0], mean)
			# if the distance is smaller than the current distance,
			# then update the bookkeeping variable
            if d < minDist[0]:
                minDist = (d, i)

        if self.colorNames[minDist[1]] == "Blue":
            color_index = 0
        elif self.colorNames[minDist[1]] == "Red":
            color_index = 1
        elif self.colorNames[minDist[1]] == "Yellow":
            color_index = 2
        elif self.colorNames[minDist[1]] == "Cyan":
            color_index = 3
        elif self.colorNames[minDist[1]] == "Green":
            color_index = 4

		# return the name of the color with the smallest distance
        return self.colorNames[minDist[1]], color_index
